      SUBROUTINE PDSYEV1( UPLO, N, A, IA, JA, DESCA, W,
     $                    WORK, LWORK, INFO )
*
* Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IA, INFO, JA, LWORK, N
*
* Array Arguments ..
      INTEGER            DESCA( * )
      DOUBLE PRECISION   A( * ), W( * ), WORK( * )
*
* Parameters ..
      INTEGER            BLOCK_CYCLIC_2D, DLEN_, DTYPE_, CTXT_, M_, N_,
     $                   MB_, NB_, RSRC_, CSRC_, LLD_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DTYPE_ = 1,
     $                   CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                   RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ZERO = 0.0D+0, ONE = 1.0D+0 )
      INTEGER            ITHVAL
      PARAMETER          ( ITHVAL = 10 )
*
* Local Scalars ..
      LOGICAL            LOWER
      INTEGER            CSRC_A, I, IACOL, IAROW, ICOFFA,
     $                   IINFO, INDD, INDD2, INDE, INDE2, INDTAU,
     $                   INDWORK, IROFFA, ISCALE,
     $                   J, K, LLWORK, LWMIN, MB_A,
     $                   MYCOL, MYROW, NB, NB_A,
     $                   NP, NPCOL, NPROCS, NPROW, NQ,
     $                   RSRC_A, SIZESYTRD
      DOUBLE PRECISION   ANRM, BIGNUM, EPS, RMAX, RMIN, SAFMIN, SIGMA,
     $                   SMLNUM
*
* Local Arrays ..
      INTEGER            IDUM1( 2 ), IDUM2( 2 )
*
* External Functions ..
      LOGICAL            LSAME
      INTEGER            INDXG2P, NUMROC
      DOUBLE PRECISION   PDLAMCH, PDLANSY
      EXTERNAL           LSAME, NUMROC, PDLAMCH, PDLANSY
*
* External Subroutines ..
      EXTERNAL           BLACS_GRIDEXIT, BLACS_GRIDINFO, CHK1MAT, DCOPY,
     $                   DESCINIT, DSCAL, DSTEQR2, PCHK1MAT,
     $                   PDELGET, PDLASCL, PDLASET,
     $                   PDSYTRD, PXERBLA
*
* Intrinsic Functions ..
      INTRINSIC          ABS, DBLE, ICHAR, MAX, MIN, MOD, SQRT
*
* Executable Statements ..
*
* Quick return
*
      IF( N.EQ.0 ) RETURN
*
* Test the input arguments.
*
      CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYROW, MYCOL )
      INFO = 0
*
      IF( NPROW.EQ.-1 ) THEN
         INFO = -( 600+CTXT_ )
      ELSE
      END IF
      IF( INFO .EQ. 0 ) THEN
         CALL CHK1MAT( N, 2, N, 2, IA, JA, DESCA, 6, INFO )
*
         IF( INFO.EQ.0 ) THEN
*
* Get machine constants.
*
            SAFMIN = PDLAMCH( DESCA( CTXT_ ), 'Safe minimum' )
            EPS = PDLAMCH( DESCA( CTXT_ ), 'Precision' )
            SMLNUM = SAFMIN / EPS
            BIGNUM = ONE / SMLNUM
            RMIN = SQRT( SMLNUM )
            RMAX = MIN( SQRT( BIGNUM ), ONE / SQRT( SQRT( SAFMIN ) ) )
*
            NPROCS = NPROW*NPCOL
            NB_A = DESCA( NB_ )
            MB_A = DESCA( MB_ )
            NB = NB_A
            LOWER = LSAME( UPLO, 'L' )
*
            RSRC_A = DESCA( RSRC_ )
            CSRC_A = DESCA( CSRC_ )
            IROFFA = MOD( IA-1, MB_A )
            ICOFFA = MOD( JA-1, NB_A )
            IAROW = INDXG2P( 1, NB_A, MYROW, RSRC_A, NPROW )
            IACOL = INDXG2P( 1, MB_A, MYCOL, CSRC_A, NPCOL )
            NP = NUMROC( N+IROFFA, NB, MYROW, IAROW, NPROW )
            NQ = NUMROC( N+ICOFFA, NB, MYCOL, IACOL, NPCOL )
*
           SIZESYTRD = MAX( NB * ( NP +1 ), 3 * NB )
*
*  Set up pointers into the WORK array
*
            INDTAU = 1
            INDE = INDTAU + N
            INDD = INDE + N
            INDD2 = INDD + N
            INDE2 = INDD2 + N
            INDWORK = INDE2 + N
            LLWORK = LWORK - INDWORK + 1
*
*  Compute the total amount of space needed
*
***
            LWMIN = 5*N + SIZESYTRD + 1
*
         END IF
*
         IF( INFO.EQ.0 ) THEN
            IF( .NOT.( LOWER .OR. LSAME( UPLO, 'U' ) ) ) THEN
               INFO = -1
            ELSE IF( LWORK.LT.LWMIN .AND. LWORK.NE.-1 ) THEN
               INFO = -9
            ELSE IF( IROFFA.NE.0 ) THEN
               INFO = -4
            ELSE IF( DESCA( MB_ ).NE.DESCA( NB_ ) ) THEN
               INFO = -( 600+NB_ )
            END IF
         END IF
*
         IF( LOWER ) THEN
            IDUM1( 1 ) = ICHAR( 'L' )
         ELSE
            IDUM1( 1 ) = ICHAR( 'U' )
         END IF
         IDUM2( 1 ) = 1
         IF( LWORK.EQ.-1 ) THEN
            IDUM1( 2 ) = -1
         ELSE
            IDUM1( 2 ) = 1
         END IF
         IDUM2( 2 ) = 2
            CALL PCHK1MAT( N, 2, N, 2, IA, JA, DESCA, 6, 2, IDUM1,
     $                     IDUM2, INFO )
*
* Write the required workspace for lwork queries.
*
         WORK( 1 ) = DBLE( LWMIN )
      END IF
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( DESCA( CTXT_ ), 'PDSYEV1', -INFO )
         RETURN
      ELSE IF( LWORK .EQ. -1 ) THEN
         RETURN
      END IF
*
* Scale matrix to allowable range, if necessary.
*
      ISCALE = 0
*
      ANRM = PDLANSY( 'M', UPLO, N, A, IA, JA, DESCA, WORK( INDWORK ) )
*
      IF( ANRM.GT.ZERO .AND. ANRM.LT.RMIN ) THEN
         ISCALE = 1
         SIGMA = RMIN / ANRM
      ELSE IF( ANRM.GT.RMAX ) THEN
         ISCALE = 1
         SIGMA = RMAX / ANRM
      END IF
*
      IF( ISCALE.EQ.1 ) THEN
         CALL PDLASCL( UPLO, ONE, SIGMA, N, N, A, IA, JA, DESCA, IINFO )
      END IF
*
* Reduce symmetric matrix to tridiagonal form.
*
      CALL PDSYTRD( UPLO, N, A, IA, JA, DESCA, WORK( INDD ),
     $              WORK( INDE ), WORK( INDTAU ), WORK( INDWORK ),
     $              LLWORK, IINFO )
*
* Copy the values of D, E to all processes.
*
      DO 10 I=1,N
         CALL PDELGET( 'A', ' ', WORK(INDD2+I-1), A,
     $                 I+IA-1, I+JA-1, DESCA )
 10   CONTINUE
      IF( LSAME( UPLO, 'U') ) THEN
          DO 20 I=1,N-1
             CALL PDELGET( 'A', ' ', WORK(INDE2+I-1), A, 
     $                     I+IA-1, I+JA, DESCA )
 20       CONTINUE
      ELSE
          DO 30 I=1,N-1
             CALL PDELGET( 'A', ' ', WORK(INDE2+I-1), A,
     $                     I+IA, I+JA-1, DESCA )
 30       CONTINUE
      ENDIF
*
*       DSTEQR2 is a modified version of LAPACK's DSTEQR.  The
*       modifications allow each process to perform partial updates
*       to matrix Q.
*
         CALL DSTEQR2( 'N', N, WORK( INDD2 ), WORK( INDE2 ),
     $                 WORK( INDWORK ), 1, 1, WORK( INDWORK ),
     $                 INFO )
*
* Copy eigenvalues from workspace to output array
*
      CALL DCOPY( N, WORK( INDD2 ), 1, W, 1 )
*
* If matrix was scaled, then rescale eigenvalues appropriately.
*
      IF( ISCALE .EQ. 1 ) THEN
         CALL DSCAL( N, ONE / SIGMA, W, 1 )
      END IF
*
* Compare every ith eigenvalue, or all if there are only a few,
* across the process grid to check for heterogeneity.
*
      IF( N.LE.ITHVAL ) THEN
         J = N
         K = 1
      ELSE
         J = N/ITHVAL
         K = ITHVAL
      END IF
*
      DO 40 I = 1, J
         WORK( I+INDTAU ) = W( (I-1)*K+1 )
         WORK( I+INDE ) = W( (I-1)*K+1 )
 40   CONTINUE
*
      CALL DGAMN2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDTAU ),
     $              J, 1, 1, -1, -1, 0 )
      CALL DGAMX2D( DESCA( CTXT_ ), 'a', ' ', J, 1, WORK( 1+INDE ),
     $              J, 1, 1, -1, -1, 0 )
*
      DO 50 I = 1, J
         IF( INFO.EQ.0 .AND. ( WORK( I+INDTAU )-WORK( I+INDE )
     $        .NE. ZERO ) )THEN 
            INFO = N+1
         END IF
 50   CONTINUE
*
      RETURN
*
*     End of PDSYEV1
*
      END
